<?php

namespace XenConcept\UserActivity\Widget;

use XF\Widget\AbstractWidget;

class ForumWatchers extends AbstractWidget
{
    protected $nodeId;

    public function render()
    {
        $session = \XF::session();
        if (!empty($session['robot']))
        {
            return '';
        }

        /** @var \XenConcept\UserActivity\XF\Entity\Forum $forum */
        $forum  = $this->contextParams['forum'];
        $options = \XF::options();

        if (!$forum->canViewWatchersForum())
        {
            return '';
        }

        $this->nodeId = $forum->node_id;

        $watchersParams = $this->runAndGetResult($options);

        $title = \XF::phrase('xc_user_activity_who_watch_this_forum_total_members_x', ['total' => $watchersParams['totalWatchers']]);
        if (!$watchersParams['countWatchers'])
        {
            return '';
        }

        $viewParams = [
                'forum' => $forum,
                'title'  => $title,
                'userDisplay' => $options->xc_user_activity_forum_display_users_watched,
            ] + $watchersParams;
        return $this->renderer('xc_user_activity_forum_watchers_widget', $viewParams);
    }

    protected function runAndGetResult($options)
    {
        $forumRepo = $this->getForumRepo();
        $forumWatchFinder = $forumRepo->findForumWatchersByNodeId($this->nodeId);

        $watchersLimit = $options->xc_user_activity_forum_watchers_limit;
        if ($watchersLimit)
        {
            $forumWatchers = $forumWatchFinder->limit($watchersLimit)->fetch();
        }
        else
        {
            $forumWatchers = $forumWatchFinder->fetch();
        }

        return [
            'totalWatchers'  => $forumWatchFinder->total(),
            'threadWatchers' => $forumWatchers,
            'countWatchers'  => $forumWatchers->count()
        ];
    }

    /**
     * @return string|null
     */
    public function getOptionsTemplate()
    {
        return 'admin:xc_user_activity_widget_def_options_' . $this->widgetConfig->definitionId;
    }

    /**
     * @return \XenConcept\UserActivity\XF\Repository\Forum
     */
    protected function getForumRepo()
    {
        return $this->repository('XF:Forum');
    }
}