<?php

namespace XenConcept\UserActivity\Widget;

use XF\Widget\AbstractWidget;

class ThreadWatchers extends AbstractWidget
{
    protected $threadId;
    protected $userId;

    public function render()
    {
        $session = \XF::session();
        if (!empty($session['robot']))
        {
            return '';
        }

        /** @var \XenConcept\UserActivity\XF\Entity\Thread $thread */
        $thread  = $this->contextParams['thread'];
        $options = \XF::options();

        if (!$thread->canViewThreadWatchers())
        {
            return '';
        }

        $this->threadId = $thread->thread_id;
        $this->userId = $thread->user_id;

        $watchersParams = $this->runAndGetResult($options);

        $title = \XF::phrase('xc_user_activity_who_watch_this_thread_total_members_x', ['total' => $watchersParams['totalWatchers']]);

        if (!$watchersParams['countWatchers'])
        {
            return '';
        }

        $viewParams = [
                'thread' => $thread,
                'title'  => $title,
                'userDisplay' => $options->xc_user_activity_thread_display_users_watched,
            ] + $watchersParams;
        return $this->renderer('xc_user_activity_thread_watchers_widget', $viewParams);
    }

    protected function runAndGetResult($options)
    {
        $threadRepo = $this->getThreadRepo();
        $threadWatchFinder = $threadRepo->findThreadWatchersByThreadId($this->threadId, $this->userId);

        $watchersLimit = $options->xc_user_activity_thread_watchers_limit ;
        if ($watchersLimit)
        {
            $threadWatchers = $threadWatchFinder->limit($watchersLimit)->fetch();
        }
        else
        {
            $threadWatchers = $threadWatchFinder->fetch();
        }

        return [
            'totalWatchers'  => $threadWatchFinder->total(),
            'threadWatchers' => $threadWatchers,
            'countWatchers'  => $threadWatchers->count()
        ];
    }

    /**
     * @return string|null
     */
    public function getOptionsTemplate()
    {
        return 'admin:xc_user_activity_widget_def_options_' . $this->widgetConfig->definitionId;
    }

    /**
     * @return \XenConcept\UserActivity\XF\Repository\Thread
     */
    protected function getThreadRepo()
    {
        return $this->repository('XF:Thread');
    }
}